"""Module for particle segmentation advanced settings UI."""
# Form implementation generated from reading ui file 'runWindow_PartSeg3D_advanced.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.

from PyQt6 import QtCore, QtGui, QtWidgets


class UI_ParticleSeg3DAdvanced:
    """UI for the Particle Segmentation 3D Advanced Settings Window."""

    def setupUi(self, Dialog):
        """Set up the UI components for the Particle Segmentation 3D Advanced Settings Window."""
        Dialog.setObjectName("Dialog")
        Dialog.resize(309, 383)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setGeometry(QtCore.QRect(100, 350, 201, 32))
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(
            QtWidgets.QDialogButtonBox.StandardButton.Cancel
            | QtWidgets.QDialogButtonBox.StandardButton.Ok
        )
        self.buttonBox.setObjectName("buttonBox")
        self.line = QtWidgets.QFrame(parent=Dialog)
        self.line.setGeometry(QtCore.QRect(10, 170, 291, 20))
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.widget = QtWidgets.QWidget(parent=Dialog)
        self.widget.setGeometry(QtCore.QRect(10, 20, 291, 153))
        self.widget.setObjectName("widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_3 = QtWidgets.QLabel(parent=self.widget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout.addWidget(self.label_3)
        self.lineEdit = QtWidgets.QLineEdit(parent=self.widget)
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout.addWidget(self.lineEdit)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(parent=self.widget)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.spinBox_4 = QtWidgets.QSpinBox(parent=self.widget)
        self.spinBox_4.setObjectName("spinBox_4")
        self.horizontalLayout_5.addWidget(self.spinBox_4)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_7 = QtWidgets.QLabel(parent=self.widget)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_4.addWidget(self.label_7)
        self.spinBox_6 = QtWidgets.QSpinBox(parent=self.widget)
        self.spinBox_6.setObjectName("spinBox_6")
        self.horizontalLayout_4.addWidget(self.spinBox_6)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_8 = QtWidgets.QLabel(parent=self.widget)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_3.addWidget(self.label_8)
        self.spinBox_7 = QtWidgets.QSpinBox(parent=self.widget)
        self.spinBox_7.setObjectName("spinBox_7")
        self.horizontalLayout_3.addWidget(self.spinBox_7)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_9 = QtWidgets.QLabel(parent=self.widget)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_7.addWidget(self.label_9)
        self.commandLinkButton_2 = QtWidgets.QCommandLinkButton(parent=self.widget)
        self.commandLinkButton_2.setObjectName("commandLinkButton_2")
        self.horizontalLayout_7.addWidget(self.commandLinkButton_2)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.widget1 = QtWidgets.QWidget(parent=Dialog)
        self.widget1.setGeometry(QtCore.QRect(10, 190, 291, 151))
        self.widget1.setObjectName("widget1")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget1)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.checkBox = QtWidgets.QCheckBox(parent=self.widget1)
        font = QtGui.QFont()
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.checkBox.setFont(font)
        self.checkBox.setObjectName("checkBox")
        self.verticalLayout_2.addWidget(self.checkBox)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_5 = QtWidgets.QLabel(parent=self.widget1)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_6.addWidget(self.label_5)
        self.commandLinkButton = QtWidgets.QCommandLinkButton(parent=self.widget1)
        self.commandLinkButton.setObjectName("commandLinkButton")
        self.horizontalLayout_6.addWidget(self.commandLinkButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(parent=self.widget1)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.spinBox_2 = QtWidgets.QSpinBox(parent=self.widget1)
        self.spinBox_2.setObjectName("spinBox_2")
        self.horizontalLayout_2.addWidget(self.spinBox_2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_10 = QtWidgets.QLabel(parent=self.widget1)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_9.addWidget(self.label_10)
        self.lineEdit_2 = QtWidgets.QLineEdit(parent=self.widget1)
        self.lineEdit_2.setObjectName("lineEdit_2")
        self.horizontalLayout_9.addWidget(self.lineEdit_2)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_9)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)  # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject)  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        """Set the text for the UI components."""
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("UI_ParticleSeg3DAdvanced", "Dialog"))
        self.label_3.setText(_translate("UI_ParticleSeg3DAdvanced", "Fold:"))
        self.label_4.setText(_translate("UI_ParticleSeg3DAdvanced", "Batch size"))
        self.label_7.setText(_translate("UI_ParticleSeg3DAdvanced", "Processes"))
        self.label_8.setText(
            _translate("UI_ParticleSeg3DAdvanced", "MinRelParticleSize")
        )
        self.label_9.setText(
            _translate("UI_ParticleSeg3DAdvanced", "Default model directory")
        )
        self.commandLinkButton_2.setText(
            _translate("UI_ParticleSeg3DAdvanced", "Choose folder")
        )
        self.checkBox.setText(
            _translate("UI_ParticleSeg3DAdvanced", "Train a new model")
        )
        self.label_5.setText(
            _translate("UI_ParticleSeg3DAdvanced", "Segmentated image")
        )
        self.commandLinkButton.setText(
            _translate("UI_ParticleSeg3DAdvanced", "Choose folder")
        )
        self.label_2.setText(_translate("UI_ParticleSeg3DAdvanced", "Z score"))
        self.label_10.setText(_translate("UI_ParticleSeg3DAdvanced", "New model name"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = UI_ParticleSeg3DAdvanced()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec())
