"""Module for the UI of the ExportPropertiesDialog."""
# Form implementation generated from reading ui file 'Window_exportProperties_Nov24.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets

# ## Name to function mapping
# input
# spinBox_5 = voxel_size
# spinBox_6 = background
# spinBox_8 = feret_angle
# spinBox_7 = mesh_spacing
# spinBox_9 = start_slice
# spinBox_10 = end_slice
# spinBox_3 = remove_small
# spinBox_4 = num_threads

# checkBox_24 = save_labels
# checkBox_5 = inclusions

# output
# checkBox_34 = inertia
# checkBox_35 = ferets
# checkBox_27 = entropy
# checkBox_29 = mean_max
# checkBox_21 = aspect_ratio
# checkBox_33 = moments
# checkBox_36 = euler
# checkBox_31 = solidity

# checkBox_37 = histograms
# checkBox_38 = basic_properties
# checkBox_23 = select_all

# buttons
# pushButton = subdataset
# pushButton_2 = all_data
# pushButton_3 = cancel


class ExportPropertiesDialog:
    """Class for the Export Properties Dialog UI."""

    def setupUi(self, Dialog):
        """Set up the UI for the Export Properties Dialog."""
        Dialog.setObjectName("Dialog")
        Dialog.resize(485, 284)
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(parent=Dialog)
        self.verticalLayoutWidget_2.setGeometry(QtCore.QRect(10, 10, 471, 275))
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout()
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_9 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_9.setToolTip("")
        self.label_9.setWhatsThis("")
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_12.addWidget(self.label_9)
        self.spinBox_5 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_5.setMouseTracking(False)
        self.spinBox_5.setToolTip("")
        self.spinBox_5.setToolTipDuration(20)
        self.spinBox_5.setStatusTip("")
        self.spinBox_5.setWhatsThis("")
        self.spinBox_5.setMaximum(1000)
        self.spinBox_5.setProperty("value", 5)
        self.spinBox_5.setObjectName("spinBox_5")
        self.horizontalLayout_12.addWidget(self.spinBox_5)
        self.verticalLayout_12.addLayout(self.horizontalLayout_12)
        self.verticalLayout_7.addLayout(self.verticalLayout_12)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_10 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_13.addWidget(self.label_10)
        self.spinBox_6 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_6.setMaximum(30000)
        self.spinBox_6.setSingleStep(100)
        self.spinBox_6.setProperty("value", 500)
        self.spinBox_6.setObjectName("spinBox_6")
        self.horizontalLayout_13.addWidget(self.spinBox_6)
        self.verticalLayout_13.addLayout(self.horizontalLayout_13)
        self.verticalLayout_7.addLayout(self.verticalLayout_13)
        self.verticalLayout_15 = QtWidgets.QVBoxLayout()
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label_12 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_15.addWidget(self.label_12)
        self.spinBox_8 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_8.setMaximum(60)
        self.spinBox_8.setSingleStep(2)
        self.spinBox_8.setProperty("value", 10)
        self.spinBox_8.setObjectName("spinBox_8")
        self.horizontalLayout_15.addWidget(self.spinBox_8)
        self.verticalLayout_15.addLayout(self.horizontalLayout_15)
        self.verticalLayout_7.addLayout(self.verticalLayout_15)
        self.verticalLayout_14 = QtWidgets.QVBoxLayout()
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_11 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_14.addWidget(self.label_11)
        self.spinBox_7 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_7.setMaximum(10)
        self.spinBox_7.setProperty("value", 1)
        self.spinBox_7.setObjectName("spinBox_7")
        self.horizontalLayout_14.addWidget(self.spinBox_7)
        self.verticalLayout_14.addLayout(self.horizontalLayout_14)
        self.verticalLayout_7.addLayout(self.verticalLayout_14)
        self.verticalLayout.addLayout(self.verticalLayout_7)
        self.horizontalLayout.addLayout(self.verticalLayout)
        spacerItem = QtWidgets.QSpacerItem(
            15,
            20,
            QtWidgets.QSizePolicy.Policy.Fixed,
            QtWidgets.QSizePolicy.Policy.Minimum,
        )
        self.horizontalLayout.addItem(spacerItem)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.label_14 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_14.setToolTip("")
        self.label_14.setWhatsThis("")
        self.label_14.setObjectName("label_14")
        self.verticalLayout_17.addWidget(self.label_14)
        self.label_13 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_13.setToolTip("")
        self.label_13.setWhatsThis("")
        self.label_13.setObjectName("label_13")
        self.verticalLayout_17.addWidget(self.label_13)
        self.horizontalLayout_17.addLayout(self.verticalLayout_17)
        self.horizontalLayout_18.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout()
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.spinBox_9 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_9.setMaximum(10000)
        self.spinBox_9.setObjectName("spinBox_9")
        self.spinBox_9.setMinimum(-1)
        self.spinBox_9.setProperty("value", -1)
        self.verticalLayout_16.addWidget(self.spinBox_9)
        self.spinBox_10 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_10.setMaximum(10000)
        self.spinBox_10.setObjectName("spinBox_10")
        self.spinBox_10.setMinimum(-1)
        self.spinBox_10.setProperty("value", -1)
        self.verticalLayout_16.addWidget(self.spinBox_10)
        self.horizontalLayout_16.addLayout(self.verticalLayout_16)
        self.horizontalLayout_18.addLayout(self.horizontalLayout_16)
        self.verticalLayout_4.addLayout(self.horizontalLayout_18)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.checkBox_24 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_24.setObjectName("checkBox_24")
        self.horizontalLayout_3.addWidget(self.checkBox_24)
        # self.checkBox_5 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        # self.checkBox_5.setObjectName("checkBox_5")
        # self.horizontalLayout_3.addWidget(self.checkBox_5)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.spinBox_3 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_3.setMaximum(10000)
        self.spinBox_3.setProperty("value", 1000)
        self.spinBox_3.setObjectName("spinBox_3")
        self.horizontalLayout_8.addWidget(self.spinBox_3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_7 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_9.addWidget(self.label_7)
        self.spinBox_4 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_4.setMinimum(-1)
        self.spinBox_4.setMaximum(50)
        self.spinBox_4.setProperty("value", -1)
        self.spinBox_4.setObjectName("spinBox_4")
        self.horizontalLayout_9.addWidget(self.spinBox_4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_7.addLayout(self.verticalLayout_4)
        self.horizontalLayout.addLayout(self.horizontalLayout_7)
        spacerItem1 = QtWidgets.QSpacerItem(
            15,
            20,
            QtWidgets.QSizePolicy.Policy.Fixed,
            QtWidgets.QSizePolicy.Policy.Minimum,
        )
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        # self.pushButton = QtWidgets.QPushButton(parent=self.verticalLayoutWidget_2)
        # self.pushButton.setObjectName("pushButton")
        # self.horizontalLayout_4.addWidget(self.pushButton)
        self.horizontalLayout_6.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.pushButton_2 = QtWidgets.QPushButton(parent=self.verticalLayoutWidget_2)
        self.pushButton_2.setObjectName("pushButton_2")
        self.verticalLayout_2.addWidget(self.pushButton_2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.pushButton_3 = QtWidgets.QPushButton(parent=self.verticalLayoutWidget_2)
        self.pushButton_3.setObjectName("pushButton_3")
        self.horizontalLayout_5.addWidget(self.pushButton_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.verticalLayout_5.addLayout(self.verticalLayout_2)
        self.horizontalLayout.addLayout(self.verticalLayout_5)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.line_3 = QtWidgets.QFrame(parent=self.verticalLayoutWidget_2)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_3.addWidget(self.line_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_8 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_10.addWidget(self.label_8)
        self.checkBox_34 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_34.setObjectName("checkBox_34")
        self.verticalLayout_10.addWidget(self.checkBox_34)
        self.checkBox_35 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_35.setObjectName("checkBox_35")
        self.verticalLayout_10.addWidget(self.checkBox_35)
        self.checkBox_27 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_27.setObjectName("checkBox_27")
        self.verticalLayout_10.addWidget(self.checkBox_27)
        self.checkBox_29 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_29.setObjectName("checkBox_29")
        self.verticalLayout_10.addWidget(self.checkBox_29)
        self.horizontalLayout_2.addLayout(self.verticalLayout_10)
        self.verticalLayout_20 = QtWidgets.QVBoxLayout()
        self.verticalLayout_20.setObjectName("verticalLayout_20")
        self.checkBox_21 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_21.setObjectName("checkBox_21")
        self.verticalLayout_20.addWidget(self.checkBox_21)
        self.checkBox_33 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_33.setObjectName("checkBox_33")
        self.verticalLayout_20.addWidget(self.checkBox_33)
        self.checkBox_36 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_36.setObjectName("checkBox_36")
        self.verticalLayout_20.addWidget(self.checkBox_36)
        self.checkBox_31 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_31.setObjectName("checkBox_31")
        self.verticalLayout_20.addWidget(self.checkBox_31)
        self.horizontalLayout_2.addLayout(self.verticalLayout_20)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.verticalLayout_19 = QtWidgets.QVBoxLayout()
        self.verticalLayout_19.setObjectName("verticalLayout_19")
        self.checkBox_37 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.checkBox_37.setFont(font)
        self.checkBox_37.setChecked(True)
        self.checkBox_37.setObjectName("checkBox_37")
        self.verticalLayout_19.addWidget(self.checkBox_37)
        self.checkBox_38 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.checkBox_38.setFont(font)
        self.checkBox_38.setChecked(True)
        self.checkBox_38.setObjectName("checkBox_38")
        self.verticalLayout_19.addWidget(self.checkBox_38)
        self.checkBox_23 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.checkBox_23.setFont(font)
        self.checkBox_23.setObjectName("checkBox_23")
        self.verticalLayout_19.addWidget(self.checkBox_23)
        self.horizontalLayout_19.addLayout(self.verticalLayout_19)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_19)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        """Set the text for the UI elements."""
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label.setText(_translate("Dialog", "Inputs"))
        self.label_9.setText(_translate("Dialog", "Voxel Size (um)"))
        self.label_10.setText(_translate("Dialog", "Background"))
        self.label_12.setText(_translate("Dialog", "Feret angle"))
        self.label_11.setText(_translate("Dialog", "mesh spacing"))
        self.label_14.setText(_translate("Dialog", "Start slice"))
        self.label_13.setText(_translate("Dialog", "End slice"))
        self.checkBox_24.setText(_translate("Dialog", "Save labels"))
        # self.checkBox_5.setText(_translate("Dialog", "Inclusions"))
        self.label_6.setText(_translate("Dialog", "Remove Small (vx)"))
        self.label_7.setText(_translate("Dialog", "#Treads"))
        self.pushButton_2.setText(_translate("Dialog", "extract"))
        self.pushButton_3.setText(_translate("Dialog", "Cancel"))
        self.label_8.setText(_translate("Dialog", "Outputs"))
        self.checkBox_34.setText(_translate("Dialog", "inertia"))
        self.checkBox_35.setText(_translate("Dialog", "Ferets"))
        self.checkBox_27.setText(_translate("Dialog", "Entropy"))
        self.checkBox_29.setText(_translate("Dialog", "Mean/Max"))
        self.checkBox_21.setText(_translate("Dialog", "Aspect ratio"))
        self.checkBox_33.setText(_translate("Dialog", "Moments"))
        self.checkBox_36.setText(_translate("Dialog", "Euler"))
        self.checkBox_31.setText(_translate("Dialog", "Solidity"))
        self.checkBox_37.setText(_translate("Dialog", "Histograms"))
        self.checkBox_38.setText(_translate("Dialog", "Basic properties"))
        self.checkBox_23.setText(_translate("Dialog", "Select all"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = ExportPropertiesDialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec())
